/*#############################################################
File Name       : ex3_analog_smooth_LM6029ACW.c
Author          : Grant Phillips
Date Modified   : 08/09/2013
Compiler        : Keil ARM-MDK (uVision V4.70.0.0)
Tested On       : STM32F0-Discovery

Description     : The same as ex2_analog_LM6029ACWc, but reads 
									16 samples, sums them and then calculates the 
									average result and displays this "smoothed" 
									out value.  In the previous example the 
									analog value would jump around a bit due 
									to noise.      

Requirements    : * STM32F0-Discovery Board
									* Make sure the LM6029ACW_F0.c file is in the 
                    same folder than the project
									* Define the SET and RESET functions for the
										output port pins in the LM6029ACW_F0.c
										library
									* Do the proper GPIO initialization for the
										output port pins in LM6029ACW_Init() in the
										LM6029ACW_F0.c library
              
Circuit         : * a potentiometer (POT) connected to PA1 
                    (ADC_IN1 - Analog Channel 1) 
									* The LM6029ACW is connected as described in
										ex1_LM6029ACW_HelloWorld.c

See the STM32F0-Discovery User Manual (UM1525) for the block 
diagram of the STM32F051R8T6 processor (p13), a summary of
the GPIO connections (p20-26) and the schematic diagram (p37-39)

##############################################################*/

/* #includes -------------------------------------------------*/
#include "main.h"																								//main library to include for device drivers, peripheral drivers, etc.
#include "LM6029ACW_F0.c"																				//for the LM6029ACW graphics lcd functions on the STM32F0-Discovery

/* #defines --------------------------------------------------*/

/* #function prototypes --------------------------------------*/
void Analogs_Init(void);																				//prototype for the user function to initialize the ADC

/* #global variables -----------------------------------------*/



// Unused global variables that have to be included to ensure correct compiling */
// ###### DO NOT CHANGE ######
// ===============================================================================
__IO uint32_t TimingDelay = 0;																	//used with the Delay function
__IO uint8_t DataReady = 0;
__IO uint32_t USBConnectTimeOut = 100;
__IO uint32_t UserButtonPressed = 0;
__IO uint8_t PrevXferComplete = 1;
// ===============================================================================



int main(void)
{
	RCC_ClocksTypeDef 			RCC_Clocks;														//structure used for setting up the SysTick Interrupt
	char buf[21];
	unsigned int ADC_val;																					//variable for holding the 12-bit ADC result
	unsigned int ADC_sum=0;																				//variable to hold the sum of the samples
	unsigned int ADC_ave=0;																				//variable to hold the average of the samples
	unsigned int sample_count = 0;   															//keeps track of how many samples have been read
	unsigned int ADC_voltage;																			//variable for the calculated voltage
	unsigned int dec, frac;          															//used for printing the voltage as "float"
	unsigned int ADC_slider=0;																		//variable for the calculated slider position
																																
	/* Set the SysTick Interrupt to occur every 1ms) */
	RCC_GetClocksFreq(&RCC_Clocks);
	if (SysTick_Config(RCC_Clocks.HCLK_Frequency / 1000))
		while(1);
	
	LM6029ACW_Init();												//initialize the lcd
	Analogs_Init();													//initialize the ADC for this application using 
																					//a user defined function at the end of the program 
																					//which has the same code as in ex1_analogs_leds.c
		
	LM6029ACW_DrawLine(13,63,112,63,1);			//draw the slider bottom marker
	LM6029ACW_DrawLine(13,63,13,60,1);			//draw the 0V marker
	LM6029ACW_DrawLine(63,63,63,60,1);			//draw the 1.65V marker
	LM6029ACW_DrawLine(112,63,112,60,1);		//draw the 3.3V marker
	
	/* Main program loop */
  while (1)
  {
		LM6029ACW_DrawLine(ADC_slider,58,ADC_slider,50,0);				//draw the slider in color = 0 to remove it at the old position
		
    while(ADC_GetFlagStatus(ADC1, ADC_FLAG_EOC) == RESET);		//wait for EndOfConversion
    ADC_val = ADC_GetConversionValue(ADC1);										//read the converted data
		ADC_sum = ADC_sum + ADC_val;															//add the result to the sum
		sample_count++;																						//increment the number of samples that was taken
		
		if(sample_count == 16)																		//only 16 samples can be stored in a integer.  Integer is 16bits, i.e. max value of 65535.
		{																													//Each sample can be a max value of 4095, therefore 4095 x 16 = 65535, which will just fit into a unsigned int variable
			ADC_ave = ADC_sum / 16;																	//calculate the average of the last 16 samples
			ADC_sum=0;																							//reset sum value
			sample_count=0;																					//reset sample count variable
		}
		
		sprintf(buf, "ADC value (DEC): %4d", ADC_ave);
		LM6029ACW_GotoXY(0,0);
		LM6029ACW_PutStr(buf, 1);
	  
		sprintf(buf, "ADC value (HEX): %4x", ADC_ave);
		LM6029ACW_GotoXY(0,1);
		LM6029ACW_PutStr(buf, 1);
		
		ADC_voltage = ADC_ave * 3300 / 4095;											//converts the last ADC average value from 0 to 4095, to 0 to 3300
		dec = ADC_voltage / 1000;																	//the first digit in ADC_voltage is the whole number part
		frac = ADC_voltage - (dec * 1000);												//the next 3 digits in ADC_voltage is the fraction
		sprintf(buf, "ADC voltage: %d.%03d V", dec, frac);				//%03d prints a integer value in a field of 3 characters with leading 0's
		LM6029ACW_GotoXY(0,2);
		LM6029ACW_PutStr(buf, 1);
		
		ADC_slider = 13 + (ADC_ave * 100 / 4095);									//calculate the new slider position
		LM6029ACW_DrawLine(ADC_slider,58,ADC_slider,50,1);				//draw the slider in color
		Delay(20);																								//wait before removing the slider again
	}	
}


/* User define function to setup the ADC for this application */
void Analogs_Init(void)
{
	GPIO_InitTypeDef        GPIO_InitStructure;
  ADC_InitTypeDef 				ADC_InitStructure;										//structure used for setting up the ADC
	
	__IO unsigned int ADC_calibration_value;											//variable for storing the calibration value
	
  /* ADC1 Periph clock enable */
  RCC_APB2PeriphClockCmd(RCC_APB2Periph_ADC1, ENABLE);
						
	/* ADC Channel configuration */
  /* GPIOA Periph clock enable */
  RCC_AHBPeriphClockCmd(RCC_AHBPeriph_GPIOA, ENABLE);

  /* Configure ADC1 Channel5 (PA1) as analog input */
  GPIO_InitStructure.GPIO_Pin = GPIO_Pin_1 ;
  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AN;
  GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_NOPULL ;
  GPIO_Init(GPIOA, &GPIO_InitStructure);
  
	/* Fill each ADC_InitStructure member with its default value */
  ADC_StructInit(&ADC_InitStructure);

	/* Configure the ADC1 in continous mode withe a resolutuion equal to 12 bits*/
  ADC_InitStructure.ADC_Resolution = ADC_Resolution_12b;													//12-bit resolution
  ADC_InitStructure.ADC_ContinuousConvMode = ENABLE;															//enable continous mode
  ADC_InitStructure.ADC_ExternalTrigConvEdge = ADC_ExternalTrigConvEdge_None;    	//ADC No external trigger for regular conversion
  ADC_InitStructure.ADC_ExternalTrigConv = ADC_ExternalTrigConv_T1_TRGO;					//ADC external trigger event 0
  ADC_InitStructure.ADC_DataAlign = ADC_DataAlign_Right;													//ADC Data alignment right 
  ADC_InitStructure.ADC_ScanDirection = ADC_ScanDirection_Upward;									//scan ADC channels upward
  ADC_Init(ADC1, &ADC_InitStructure);
	
	/* Convert the ADC1 Channel 1 with 7.5 Cycles as sampling time */ 
  ADC_ChannelConfig(ADC1, ADC_Channel_1 , ADC_SampleTime_7_5Cycles);

	 /* ADC Calibration */
  ADC_GetCalibrationFactor(ADC1);

  /* Enable the auto delay feature */    
  ADC_WaitModeCmd(ADC1, ENABLE); 
  
  /* Enable the Auto power off mode */
  ADC_AutoPowerOffCmd(ADC1, ENABLE); 
  
  /* Enable ADCperipheral[PerIdx] */
  ADC_Cmd(ADC1, ENABLE);     
  
  /* Wait the ADEN flag */
  while(!ADC_GetFlagStatus(ADC1, ADC_FLAG_ADEN)); 
   
  /* ADC1 regular Software Start Conv */ 
  ADC_StartOfConversion(ADC1);
}









// -------------------------------------------------------------------------------

// Function to insert a timing delay of nTime
// ###### DO NOT CHANGE ######
void Delay(__IO uint32_t nTime)
{
  TimingDelay = nTime;

  while(TimingDelay != 0);
}

// Function to Decrement the TimingDelay variable.
// ###### DO NOT CHANGE ######
void TimingDelay_Decrement(void)
{
  if (TimingDelay != 0x00)
  { 
    TimingDelay--;
  }
}

// Unused functions that have to be included to ensure correct compiling
// ###### DO NOT CHANGE ######
// =======================================================================
uint32_t L3GD20_TIMEOUT_UserCallback(void)
{
  return 0;
}

uint32_t LSM303DLHC_TIMEOUT_UserCallback(void)
{
  return 0;
}
// =======================================================================



